import "./L.VectorGrid";
import Pbf from "pbf";
import { VectorTile } from "vector-tile";
L.VectorGrid.Protobuf = L.VectorGrid.extend({
    options: {
        subdomains: "abc",
        fetchOptions: {},
    },
    initialize: function (url, options) {
        this._url = url;
        L.VectorGrid.prototype.initialize.call(this, options);
    },
    setUrl: function (url, noRedraw) {
        this._url = url;
        if (!noRedraw) {
            this.redraw();
        }
        return this;
    },
    _getSubdomain: L.TileLayer.prototype._getSubdomain,
    _getVectorTilePromise: function (coords) {
        var data = {
            s: this._getSubdomain(coords),
            x: coords.x,
            y: coords.y,
            z: coords.z,
        };
        if (this._map && !this._map.options.crs.infinite) {
            var invertedY = this._globalTileRange.max.y - coords.y;
            if (this.options.tms) {
                data["y"] = invertedY;
            }
            data["-y"] = invertedY;
        }
        var tileUrl = L.Util.template(this._url, L.extend(data, this.options));
        return fetch(tileUrl, this.options.fetchOptions)
            .then(function (response) {
            if (!response.ok) {
                return { layers: [] };
            }
            return response.blob().then(function (blob) {
                var reader = new FileReader();
                return new Promise(function (resolve) {
                    reader.addEventListener("loadend", function () {
                        var pbf = new Pbf(reader.result);
                        return resolve(new VectorTile(pbf));
                    });
                    reader.readAsArrayBuffer(blob);
                });
            });
        })
            .then(function (json) {
            for (var layerName in json.layers) {
                var feats = [];
                for (var i = 0; i < json.layers[layerName].length; i++) {
                    var feat = json.layers[layerName].feature(i);
                    feat.geometry = feat.loadGeometry();
                    feats.push(feat);
                }
                json.layers[layerName].features = feats;
            }
            console.log(json);
            return json;
        });
    },
});
var protobuf = function (url, options) {
    return new L.VectorGrid.Protobuf(url, options);
};
export default protobuf;
