import L from "leaflet";
import IBounds from "../IBounds";
import Layer from "../Layer";
export default abstract class AbstractLeafletLayer extends Layer {
    static readonly MAX_ZINDEX = 99999999999;
    static readonly MIN_ZINDEX = -99999999999;
    static readonly ZINDEX_STEP = 20;
    url: string;
    pane?: string;
    center: number[];
    zIndex: number;
    bounds: IBounds;
    subdomains: String | String[];
    filter: {
        brightness: number;
        contrast: number;
        grayscale: number;
        saturate: number;
        opacity: number;
        sepia: number;
        "hue-rotate": number;
        blur: number;
        invert: number;
    };
    constructor(id: string, url: string, pane?: string);
    setCenter(center: number[]): void;
    setFilter(filter: {
        brightness: number;
        contrast: number;
        grayscale: number;
        saturate: number;
        opacity: number;
        sepia: number;
        "hue-rotate": number;
        blur: number;
        invert: number;
    }): void;
    setBounds(bounds: IBounds): void;
    setSubdomains(domains: String | String[]): void;
    protected getZIndex(): number;
    getActualBounds(): L.LatLngBounds;
    createCoordinate(lat: number, lon: number): L.LatLng;
    getLayerOptions(): {
        id: string;
        center: number[];
        maxZoom: number;
        minZoom: number;
        bounds: L.LatLngBounds;
        filter: {
            brightness: number;
            contrast: number;
            grayscale: number;
            saturate: number;
            opacity: number;
            sepia: number;
            "hue-rotate": number;
            blur: number;
            invert: number;
        };
        zIndex: number;
        pane: string;
    } | {
        id: string;
        center: number[];
        maxZoom: number;
        minZoom: number;
        bounds: L.LatLngBounds;
        filter: {
            brightness: number;
            contrast: number;
            grayscale: number;
            saturate: number;
            opacity: number;
            sepia: number;
            "hue-rotate": number;
            blur: number;
            invert: number;
        };
        zIndex: number;
        pane?: undefined;
    };
    setMinZoom(minZoom: number): void;
    setMaxZoom(maxZoom: number): void;
    getCustomPane(): string;
    abstract getLayer(): any;
    on(eventName: string, map: any, props?: any): void;
    off(eventName: string): void;
}
