var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import L from "leaflet";
import "leaflet/dist/leaflet.css";
import sideBySide from "../../control/leaflet-side-by-side/L.Control.Sider";
import mousePosition from "../../control/mouse-position/L.Control.MousePosition";
import zoomLable from "../../control/zoom-label/L.Control.ZoomLabel";
import zoomSlider from "../../control/zoom-slider/L.Control.ZoomSlider";
import AbstractMap from "../AbstractMap";
var LeafletMap = (function (_super) {
    __extends(LeafletMap, _super);
    function LeafletMap(container, options, initZoom, initCenter) {
        if (initZoom === void 0) { initZoom = AbstractMap.INIT_ZOOM; }
        if (initCenter === void 0) { initCenter = AbstractMap.INIT_LEAFLET_CENTER; }
        var _this = _super.call(this, container, initCenter, initZoom) || this;
        _this.layers = new Map();
        _this.initMap(options);
        return _this;
    }
    LeafletMap.prototype.doInitMap = function (options) {
        var _options = this.getDefaultMapOption();
        if (options) {
            Object.assign(_options, options);
        }
        var map = L.map(this.container, _options);
        map.setView([this.getInitCenter()[0], this.getInitCenter()[1]], this.getInitZoom());
        return map;
    };
    LeafletMap.prototype.getDefaultMapOption = function () {
        return {
            zoomControl: true,
            attributionControl: false,
            maxBounds: L.latLngBounds(L.latLng(-85.06, -180), L.latLng(85.06, 180)),
            maxZoom: 18,
            minZoom: 3,
        };
    };
    LeafletMap.prototype.addMousePosition = function (position) {
        if (position === void 0) { position = LeafletMap.POSITION_BOTTOM_RIGHT; }
        if (this.hasMap()) {
            mousePosition({ position: position }).addTo(this.getMap());
        }
    };
    LeafletMap.prototype.addZoomSlider = function (position) {
        if (position === void 0) { position = LeafletMap.POSITION_TOP_LEFT; }
        if (this.hasMap()) {
            zoomSlider({ position: position }).addTo(this.getMap());
        }
    };
    LeafletMap.prototype.addZoomLable = function (position) {
        if (position === void 0) { position = LeafletMap.POSITION_TOP_LEFT; }
        if (this.hasMap()) {
            zoomLable({ position: position }).addTo(this.getMap());
        }
    };
    LeafletMap.prototype.addSideBySide = function (leafLayer, rightLayer) {
        this.sideControl = sideBySide(leafLayer.getLayer(), rightLayer.getLayer()).addTo(this.getMap());
        console.log(this.sideControl);
    };
    LeafletMap.prototype.getSideControl = function () {
        return this.sideControl;
    };
    LeafletMap.prototype.setMaxBounds = function (bounds) {
        if (this.hasMap()) {
            this.getMap().setMaxBounds([
                [bounds.minLat, bounds.minLon],
                [bounds.maxLat, bounds.maxLon],
            ]);
        }
    };
    LeafletMap.prototype.addLayer = function (layer) {
        if (this.hasMap()) {
            var ly = layer.getLayer();
            ly.setZIndex(layer.zIndex);
            this.getMap().addLayer(ly);
            this.layers.set(layer.getId(), ly);
        }
    };
    LeafletMap.prototype.addOtherLayer = function (layer, id) {
        if (this.hasMap()) {
            this.getMap().addLayer(layer);
            this.layers.set(id, layer);
        }
    };
    LeafletMap.prototype.moveLayer = function (id, beforeId) {
        if (this.layers) {
            var layer1 = this.layers.get(id);
            var layer2 = this.layers.get(beforeId);
            var zIndex1 = layer1[0].options.zIndex;
            var zIndex2 = layer2[0].options.zIndex;
            layer2[0].setZIndex(zIndex1);
            layer1[0].setZIndex(zIndex2);
        }
    };
    LeafletMap.prototype.removeLayer = function (id) {
        if (this.layers) {
            if (id) {
                var layer = this.layers.get(id);
                if (layer) {
                    this.getMap().removeLayer(layer);
                }
            }
        }
    };
    LeafletMap.prototype.getLayer = function (id) {
        if (this.layers) {
            if (id) {
                var layer = this.layers.get(id);
                return layer;
            }
        }
    };
    LeafletMap.prototype.setView = function (center, zoom, options) {
        if (this.hasMap) {
            this.getMap().setView(center, zoom, options);
        }
    };
    LeafletMap.prototype.fitBounds = function (bounds) {
        if (this.hasMap) {
            this.getMap().fitBounds(bounds);
        }
    };
    LeafletMap.prototype.close = function () {
        this.layers = undefined;
        if (this.sideControl) {
            this.sideControl.remove();
        }
        if (this.getMap()) {
            this.getMap().remove();
        }
    };
    LeafletMap.prototype.createPane = function (panelId) {
        if (this.getMap()) {
            this.getMap().createPane(panelId);
        }
    };
    LeafletMap.EVENT_CLICK = "click";
    LeafletMap.EVENT_DBCLICK = "dblclick";
    LeafletMap.EVENT_MOUSE_DOWN = "mousedown";
    LeafletMap.EVENT_MOUSE_UP = "mouseup";
    LeafletMap.EVENT_MOUSE_MOVE = "mousemove";
    LeafletMap.EVENT_ZOOM_MOVE = "zoom move";
    LeafletMap.POSITION_TOP_LEFT = "topleft";
    LeafletMap.POSITION_TOP_RIGHT = "topright";
    LeafletMap.POSITION_BOTTOM_LEFT = "bottomleft";
    LeafletMap.POSITION_BOTTOM_RIGHT = "bottomright";
    return LeafletMap;
}(AbstractMap));
export default LeafletMap;
