import L from "leaflet";
import "leaflet/dist/leaflet.css";
import WebMecatorTileUtils from "../util/WebMecatorTileUtils";
var LeafletDrawLonlatLine = (function () {
    function LeafletDrawLonlatLine(map) {
        this.lineColor = "red";
        this.textColor = 'yellow';
        this.textPosition = LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_ALL;
        this.iconStyleName = undefined;
        this.map = map;
        this.lonLatGridLineLayers = this.initLonLatGridLayers();
    }
    LeafletDrawLonlatLine.prototype.initLonLatGridLayers = function () {
        var layer = L.featureGroup().addTo(this.map);
        return layer;
    };
    LeafletDrawLonlatLine.prototype.getLonLatGridLayers = function () {
        return this.lonLatGridLineLayers;
    };
    LeafletDrawLonlatLine.prototype.setLineColor = function (lineColor) {
        this.lineColor = lineColor;
    };
    LeafletDrawLonlatLine.prototype.setTextColor = function (textColor) {
        this.textColor = textColor;
    };
    LeafletDrawLonlatLine.prototype.setTextPosition = function (textPosition) {
        this.textPosition = textPosition;
    };
    LeafletDrawLonlatLine.prototype.drawLonlatTileGrid = function () {
        var zoom = this.map.getZoom();
        var bounds = this.map.getBounds();
        var west = bounds.getWest();
        var east = bounds.getEast();
        var north = bounds.getNorth();
        var south = bounds.getSouth();
        var tileXYRangeMap = WebMecatorTileUtils.getTileXYRange(zoom, west, east, south, north);
        var minTileX = tileXYRangeMap.get("minTileX");
        var maxTileX = tileXYRangeMap.get("maxTileX");
        var minTileY = tileXYRangeMap.get("minTileY");
        var maxTileY = tileXYRangeMap.get("maxTileY");
        var xyTileRange = { zoom: zoom, minTileX: minTileX, maxTileX: maxTileX, minTileY: minTileY, maxTileY: maxTileY };
        this.drawLatTileGridLines(xyTileRange, west, east);
        this.drawLonTileGridLines(xyTileRange, south, north);
    };
    LeafletDrawLonlatLine.prototype.drawLatTileGridLines = function (xyTileRange, west, east) {
        var zoom = xyTileRange.zoom;
        for (var y = xyTileRange.minTileY; y <= xyTileRange.maxTileY; y++) {
            for (var x = xyTileRange.minTileX; x <= xyTileRange.maxTileX; x++) {
                var envelope = WebMecatorTileUtils.tileLatLonBounds(x, y, zoom);
                var maxLat = envelope.maxLat;
                var maxLatLine = L.polyline([
                    [maxLat, west],
                    [maxLat, east],
                ], { weight: 1, color: this.lineColor });
                this.lonLatGridLineLayers.addLayer(maxLatLine);
                var text = this.createLonLatText(maxLat, zoom);
                if (LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_ALL ===
                    this.textPosition ||
                    LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_RIGHT === this.textPosition) {
                    var eastDivIcon = this.createDivIcon((text.length + 1) * 6, -5, text);
                    var eastTextMarker = L.marker([maxLat, east], { icon: eastDivIcon });
                    this.lonLatGridLineLayers.addLayer(eastTextMarker);
                }
                if (LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_ALL ===
                    this.textPosition ||
                    LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_LEFT === this.textPosition) {
                    var westDivIcon = this.createDivIcon(-text.length * 1, 20, text);
                    var westTextMarker = L.marker([maxLat, west], { icon: westDivIcon });
                    this.lonLatGridLineLayers.addLayer(westTextMarker);
                }
                break;
            }
        }
    };
    LeafletDrawLonlatLine.prototype.drawLonTileGridLines = function (xyTileRange, south, north) {
        var zoom = xyTileRange.zoom;
        for (var x = xyTileRange.minTileX; x <= xyTileRange.maxTileX; x++) {
            for (var y = xyTileRange.minTileY; y <= xyTileRange.maxTileY; y++) {
                var envelope = WebMecatorTileUtils.tileLatLonBounds(x, y, zoom);
                var minLon = envelope.minLon;
                var lonLine = L.polyline([
                    [south, minLon],
                    [north, minLon],
                ], { weight: 1, color: this.lineColor });
                this.lonLatGridLineLayers.addLayer(lonLine);
                var text = this.createLonLatText(minLon, zoom);
                if (LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_ALL ===
                    this.textPosition ||
                    LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_RIGHT === this.textPosition) {
                    var northDivIcon = this.createDivIcon(-5, -5, text);
                    var northTextMarker = L.marker([north, minLon], {
                        icon: northDivIcon,
                    });
                    this.lonLatGridLineLayers.addLayer(northTextMarker);
                }
                if (LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_ALL ===
                    this.textPosition ||
                    LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_LEFT === this.textPosition) {
                    var southDivIcon = this.createDivIcon(text.length * 7, 20, text);
                    var southTextMarker = L.marker([south, minLon], {
                        icon: southDivIcon,
                    });
                    this.lonLatGridLineLayers.addLayer(southTextMarker);
                }
                break;
            }
        }
    };
    LeafletDrawLonlatLine.prototype.createDivIcon = function (tipX, tipY, text) {
        var divIcon = L.divIcon({
            className: "." + this.iconStyleName,
            html: "<div style=\"white-space: nowrap;color:".concat(this.textColor, ";\">").concat(text, "</div>"),
            iconAnchor: [tipX, tipY],
        });
        return divIcon;
    };
    LeafletDrawLonlatLine.prototype.createLonLatText = function (point, zoom) {
        var text = point.toFixed(1) + "°";
        if (zoom > 10) {
            text = point.toFixed((zoom - 8) / 2) + "°";
        }
        if (point > 0) {
            text = text + "E";
        }
        else if (point < 0) {
            text = text + "W";
        }
        return text;
    };
    LeafletDrawLonlatLine.prototype.clearLayers = function () {
        if (this.lonLatGridLineLayers) {
            this.lonLatGridLineLayers.clearLayers();
        }
    };
    LeafletDrawLonlatLine.prototype.addDrawEvent = function () {
        var _this = this;
        this.map.on("zoom move", function () {
            _this.clearLayers();
            _this.drawLonlatTileGrid();
        });
    };
    LeafletDrawLonlatLine.prototype.removeDrawEvent = function () {
        this.clearLayers();
        this.map.off("zoom move");
    };
    LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_LEFT = "left";
    LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_RIGHT = "right";
    LeafletDrawLonlatLine.LONLAT_TEXT_POSITION_ALL = "all";
    return LeafletDrawLonlatLine;
}());
export default LeafletDrawLonlatLine;
