var WebMecatorTileUtils = (function () {
    function WebMecatorTileUtils() {
    }
    WebMecatorTileUtils.getMatrixSize = function (level) {
        return 1 << level;
    };
    WebMecatorTileUtils.resolution = function (zoom) {
        return ((2 * WebMecatorTileUtils.maxX) / this.tileSize / this.getMatrixSize(zoom));
    };
    WebMecatorTileUtils.pixelsToMeters2 = function (x, y, zoom) {
        var res = this.resolution(zoom);
        var mx = x * res - (2 * Math.PI * 6378137) / 2.0;
        var my = y * res - (2 * Math.PI * 6378137) / 2.0;
        return { x: mx, y: Math.abs(my) };
    };
    WebMecatorTileUtils.tileBounds = function (x, y, zoom) {
        var xTiles = this.getMatrixSize(zoom);
        var yTiles = xTiles;
        var xTileWidth = (WebMecatorTileUtils.maxX - WebMecatorTileUtils.minX) / xTiles;
        var west = WebMecatorTileUtils.minX + x * xTileWidth;
        var east = WebMecatorTileUtils.minX + (x + 1) * xTileWidth;
        var yTileHeight = (WebMecatorTileUtils.maxX - WebMecatorTileUtils.minX) / yTiles;
        var north = WebMecatorTileUtils.maxX - y * yTileHeight;
        var south = WebMecatorTileUtils.maxX - (y + 1) * yTileHeight;
        return {
            x1: west,
            x2: east,
            y1: north,
            y2: south,
        };
    };
    WebMecatorTileUtils.meters2LatLon = function (x, y) {
        var lon = (x / ((2 * Math.PI * 6378137) / 2.0)) * 180.0;
        var lat = (y / ((2 * Math.PI * 6378137) / 2.0)) * 180.0;
        lat = (180 / Math.PI) * (2 * Math.atan(Math.exp((lat * Math.PI) / 180.0)) - Math.PI / 2.0);
        return { lat: lat, lon: lon };
    };
    WebMecatorTileUtils.tileLatLonBounds = function (x, y, zoom) {
        var envelope = this.tileBounds(x, y, zoom);
        var minLatLon = this.meters2LatLon(envelope.x1, envelope.y1);
        var maxLatLon = this.meters2LatLon(envelope.x2, envelope.y2);
        return {
            minLon: minLatLon.lon,
            maxLon: maxLatLon.lon,
            minLat: minLatLon.lat,
            maxLat: maxLatLon.lat,
        };
    };
    WebMecatorTileUtils.lonLat2Mercator = function (lon, lat) {
        var x = (lon * WebMecatorTileUtils.maxX) / 180;
        var y = Math.log(Math.tan(((90 + lat) * Math.PI) / 360)) / (Math.PI / 180);
        y = (y * WebMecatorTileUtils.maxX) / 180;
        return { x: x, y: y };
    };
    WebMecatorTileUtils.lonlat2TileXY = function (zoom, lon, lat) {
        var c = this.lonLat2Mercator(lon, lat);
        var xTiles = this.getMatrixSize(zoom);
        var yTiles = this.getMatrixSize(zoom);
        var overallWidth = WebMecatorTileUtils.maxX - WebMecatorTileUtils.minX;
        var xTileWidth = overallWidth / xTiles;
        var overallHeight = WebMecatorTileUtils.maxX - WebMecatorTileUtils.minX;
        var yTileHeight = overallHeight / yTiles;
        var distanceFromWest = c.x - WebMecatorTileUtils.minX;
        var distanceFromNorth = WebMecatorTileUtils.maxX - c.y;
        var xTileCoordinate = distanceFromWest / xTileWidth;
        if (xTileCoordinate >= xTiles) {
            xTileCoordinate = xTiles - 1;
        }
        var map = new Map();
        map.set("X", parseInt(String(xTileCoordinate)));
        var yTileCoordinate = distanceFromNorth / yTileHeight;
        if (yTileCoordinate >= yTiles) {
            yTileCoordinate = yTiles - 1;
        }
        map.set("Y", parseInt(String(yTileCoordinate)));
        return map;
    };
    WebMecatorTileUtils.getTileXYRange = function (zoom, minX, maxX, minY, maxY) {
        var minXY = this.lonlat2TileXY(zoom, minX, maxY);
        var maxXY = this.lonlat2TileXY(zoom, maxX, minY);
        var minTileX = minXY.get("X");
        var maxTileX = maxXY.get("X");
        var minTileY = minXY.get("Y");
        var maxTileY = maxXY.get("Y");
        var result = new Map();
        result.set("minTileX", minTileX);
        result.set("maxTileX", maxTileX);
        result.set("minTileY", minTileY);
        result.set("maxTileY", maxTileY);
        return result;
    };
    WebMecatorTileUtils.minX = -20037508.342789244;
    WebMecatorTileUtils.maxX = 20037508.342789244;
    WebMecatorTileUtils.tileSize = 256;
    return WebMecatorTileUtils;
}());
export default WebMecatorTileUtils;
